;+
; NAME:ncdf_ajoute
;
; PURPOSE:permet d''ajouter une variable a la suite d''un fichier net-cdf
;
; CATEGORY:bidouille NET-CDF
;
; CALLING SEQUENCE:ndf_ajoute,'nomfich',struct
; 
; INPUTS:
;        nomfich: string contenant le nom du fichier auquel il faut
;        ajouter la strucure contenant la variable et tout ce qui va
;        autour
;
;        nom: string contenant le nom de la variable
;
;        tab:le tableau contenant la variable
;
;        dimension[nbredim]: un vecteur de string contennant le nom des
;        dimensions, par ex ['x','y','time_counter']
;
; OPTIONAL INPUTS:
;	
;        attributs[nbreattributs,2]: un tableau 2d de string dont
;        chaque ligne contient le nom et la valeur de
;        l''attribut, par ex: attribut[i]=['titre','SST']
;
;	
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;
; OPTIONAL OUTPUTS:
;
; COMMON BLOCKS:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; EXAMPLE:
;
; MODIFICATION HISTORY: Sebastien Masson (smasson@lodyc.jussieu.fr)
;                       25/11/98
;-
;-----------------------------------------------------------------------------------------
PRO ncdf_ajoute, nomfich, nom, tab, dimension, attribut
;-----------------------------------------------------------------------------------------
; petite verificationb des dimension de dimensoin et attribut
;-----------------------------------------------------------------------------------------
   if (size(dimension))[0] EQ 0 then dimension = [dimension]
   if (size(attribut))[0] EQ 1 then attribut = reform(attribut, 2, 1)
;-----------------------------------------------------------------------------------------
; ouverture du fichier a completer
;-----------------------------------------------------------------------------------------
   cdfid = ncdf_open(nomfich, /write)
   contient = ncdf_inquire(cdfid)
   ncdf_control,cdfid,/redef 
;-----------------------------------------------------------------------------------------
; caracteristiques des dim du fichier
;-----------------------------------------------------------------------------------------
   dimid = findgen(contient.ndims)
   nomdim = strarr(contient.ndims)
   tailledim = lonarr(contient.ndims)
   for id = 0,contient.ndims-1 do begin
      ncdf_diminq, cdfid, id, name, value
      nomdim[id] = name
      tailledim[id] = value
   endfor
;-----------------------------------------------------------------------------------------
; on recupere la dimid des dimension existantes
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
; il faut creer une nouvelle dimension?
;-----------------------------------------------------------------------------------------
   quelledim = lonarr((size(dimension))[1])
   for i = 0, (size(dimension))[1]-1 do begin
      indice = where(strlowcase(nomdim) EQ strlowcase(dimension[i]))
      if indice[0] EQ -1 then BEGIN 
         dimid=[dimid,ncdf_dimdef(cdfid,dimension[i],(size(tab))[i+1])] 
         quelledim[i] = n_elements(dimid)-1 
      ENDIF ELSE quelledim[i] = indice[0]
   endfor
;-----------------------------------------------------------------------------------------
; declaration de la nouvelle variable
;-----------------------------------------------------------------------------------------
   varid = ncdf_vardef(cdfid, nom, dimid[quelledim], /float)
;-----------------------------------------------------------------------------------------
; declaration des attributs de la nouvelle variable
;-----------------------------------------------------------------------------------------
   if n_elements(attribut) NE 0 then $
   for i = 0, (size(attribut))[2]-1 do begin
      ncdf_attput, cdfid,nom, attribut[0, i], attribut[1, i]
   endfor
;------------------------------------------------------------
; fin de la redefinition du fichier
;------------------------------------------------------------
   ncdf_control,  cdfid, /endef
;------------------------------------------------------------
; remplissage de la variable
;------------------------------------------------------------
   ncdf_varput, cdfid, nom, tab
;------------------------------------------------------------
; fermeture du fichier
;------------------------------------------------------------
   ncdf_close, cdfid




;-----------------------------------------------------------------------------------------
   return
end
